/**
 * Copyright (C) 2014 Couchbase, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALING
 * IN THE SOFTWARE.
 */
package com.couchbase.client.core.config;

import com.couchbase.client.deps.com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import java.net.InetAddress;

/**
 * A configuration representing the couchbase bucket.
 */
@JsonDeserialize(as = DefaultCouchbaseBucketConfig.class)
public interface CouchbaseBucketConfig extends BucketConfig {

    /**
     * Returns the node index for the given partition index and master.
     *
     * @param partition the index of the partition.
     * @return the index of the node.
     */
    short nodeIndexForMaster(int partition);

    /**
     * Returns the node index for the given partition index and the replica.
     *
     * @param partition the index of the partition.
     * @param replica the replica number.
     * @return the index of the node.
     */
    short nodeIndexForReplica(int partition, int replica);

    /**
     * Returns the total number of partitions.
     *
     * @return the number of partitions.
     */
    int numberOfPartitions();

    /**
     * Returns information for the node at the given index.
     *
     * @param nodeIndex the index of the node.
     * @return the information of the node at this index.
     */
    NodeInfo nodeAtIndex(int nodeIndex);

    /**
     * The number of configured replicas for this bucket.
     *
     * @return number of replicas.
     */
    int numberOfReplicas();

    /**
     * Checks if the given hostname has active primary partitions assigned to it.
     *
     * @param hostname the hostname of the node to check against.
     * @return true if it has, false otherwise.
     */
    boolean hasPrimaryPartitionsOnNode(InetAddress hostname);
}
